import { app } from '@/app.ts';
import { Config } from './model/config.ts';
import { getConfig, writeConfig } from '@/module/get-config.ts';
import { queryMe } from '@/query/index.ts';
const cacheToken = 'tokenList';
export type TokenCacheItem = {
  baseURL?: string;
  token?: string;
  expireTime?: number;
};

app
  .route({
    path: 'config',
    key: 'getTokenList',
    description: 'Get token list',
  })
  .define(async (ctx) => {
    const tokenList = await Config.findOne({
      where: {
        key: cacheToken,
      },
      logging: false,
    });
    if (tokenList) {
      ctx.body = tokenList;
      return;
    }
    ctx.body = {
      value: [],
    };
  })
  .addTo(app);

app
  .route({
    path: 'config',
    key: 'setTokenList',
    description: 'Set token list',
  })
  .define(async (ctx) => {
    const { data } = ctx.query;
    if (!data) {
      ctx.throw(400, 'data is required');
    }
    let config = await Config.findOne({
      where: { key: cacheToken }, // 自定义条件
      logging: false,
    });

    if (!config) {
      config = await Config.create(
        {
          key: cacheToken,
          value: data,
        },
        { logging: false },
      );
      ctx.body = config;
      return;
    } else {
      config.value = data;
      await config.save();
      ctx.body = config;
    }
  })
  .addTo(app);

app
  .route({
    path: 'config',
    key: 'clearToken',
    description: 'Clear token list',
  })
  .define(async (ctx) => {
    const config = await Config.findOne({
      where: { key: cacheToken },
      logging: false,
    });
    if (config) {
      await config.destroy();
    }
    ctx.body = 'success';
  })
  .addTo(app);

app
  .route({
    path: 'config',
    key: 'saveToken',
    description: 'Add token',
    validator: {
      baseURL: {
        type: 'string',
        required: true,
        message: 'baseURL is required',
      },
      token: {
        type: 'string',
        required: true,
        message: 'token is required',
      },
    },
  })
  .define(async (ctx) => {
    const { baseURL, token } = ctx.query;
    if (!baseURL || !token) {
      ctx.throw(400, 'baseURL and token are required');
    }
    const data: TokenCacheItem = {
      baseURL,
      token,
    };
    const tokenRes = await ctx.call({ path: 'config', key: 'getTokenList' });
    if (tokenRes.code !== 200) {
      ctx.throw(tokenRes.code, tokenRes.message || 'Failed to get token list');
    }
    const tokenList: TokenCacheItem[] = tokenRes.body?.value || [];
    // Check if the token already exists
    const index = tokenList.findIndex((item) => item.baseURL === data.baseURL);
    if (index > -1) {
      tokenList[index] = data;
    } else {
      tokenList.push(data);
    }
    const res = await ctx.call({ path: 'config', key: 'setTokenList', payload: { data: tokenList } });
    if (res.code === 200) {
      ctx.body = res.body?.value;
    } else ctx.throw(res.code, res.message || 'Failed to add token');
  })
  .addTo(app);

app
  .route({
    path: 'config',
    key: 'switchToken',
    description: 'Switch token user',
    validator: {
      baseURL: {
        type: 'string',
        required: true,
        message: 'baseURL is required',
      },
    },
  })
  .define(async (ctx) => {
    const { baseURL } = ctx.query;
    const configRes = await ctx.call({ path: 'config', key: 'getTokenList' });
    if (configRes.code !== 200) {
      ctx.throw(configRes.code, configRes.message || 'Failed to get token list');
    }
    const tokenList: TokenCacheItem[] = configRes.body?.value || [];
    const index = tokenList.findIndex((item) => item.baseURL === baseURL);
    const token = index > -1 ? tokenList[index].token : '';
    if (token) {
      const config = getConfig();
      const resMe = await queryMe();
      if (resMe.code !== 200) {
        writeConfig({ ...config, token: '' });
        ctx.throw(resMe.code, resMe.message || 'cache token is invalid');
      }
      writeConfig({ ...config, token: token });
      ctx.body = {
        baseURL: baseURL,
        token: tokenList[index].token,
      };
    } else {
      writeConfig({ ...getConfig(), token: '' });
      ctx.throw(404, 'Token not found');
    }
  })
  .addTo(app);

app
  .route({
    path: 'config',
    key: 'deleteToken',
    description: 'Delete token',
    validator: {
      baseURL: {
        type: 'string',
        required: true,
        message: 'baseURL is required',
      },
    },
  })
  .define(async (ctx) => {
    const { baseURL } = ctx.query;
    const config = await ctx.call({ path: 'config', key: 'getTokenList' });
    if (config.code !== 200) {
      ctx.throw(config.code, config.message || 'Failed to get token list');
    }
    const tokenList: TokenCacheItem[] = config.body?.value || [];
    const index = tokenList.findIndex((item) => item.baseURL === baseURL);
    if (index > -1) {
      tokenList.splice(index, 1);
      const res = await ctx.call({ path: 'config', key: 'setTokenList', payload: { data: tokenList } });
      if (res.code === 200) {
        ctx.body = res.body;
      } else ctx.throw(res.code, res.message || 'Failed to delete token');
    } else {
      console.log('not has token', baseURL);
      ctx.body = {
        value: tokenList,
      };
    }
  })
  .addTo(app);
